{% extends 'base_nav.html' %}

{% load i18n %}
{% load static %}

{% block main_content %}

	<div class='profile profile-anon'>
		<div class='profile-org-top-cont profile-anon-top-cont'>
			<div class='profile-org-top-info-bar profile-anon-top-info-bar'>
						<h2 class='profile-name'>{{ request.session.wallet_owner }} ({{ request.session.wallet_name }})</h2>
				<div class='profile-top-bar-right'>
					<div class='btn btn-connect'><i class="fas fa-user-plus"></i>Connect</div>
					<div class='btn btn-connect'><i class="fas fa-comment"></i>Message</div>
				</div>
			</div>
		</div>

		<div class='profile-bottom-cont profile-anon-bottom-cont'>
			<div class='profile-nav-cont profile-anon-nav-cont'>
				<div class='profile-nav'>
					<div class='profile-nav-btn profile-nav-btn-profile active-profile-nav'>
						<a href="{% url 'indy_profile' %}"><h5>Profile</h5><i class="fas fa-plus"></i></a>
					</div>
					<div class='profile-nav-btn profile-nav-btn-data'>
						<a href="{% url 'indy_data' %}"><h5>Application-Managed Data</h5><i class="fas fa-plus"></i></a>
					</div>
					<div class='profile-nav-btn profile-nav-btn-connections'>
						<a href="{% url 'connections' %}"><h5>Connections</h5><i class="fas fa-plus"></i></a>
					</div>
					<div class='profile-nav-btn profile-nav-btn-conversations'>
						<a href="{% url 'conversations' %}"><h5>Conversations</h5><i class="fas fa-plus"></i></a>
					</div>
					<div class='profile-nav-btn profile-nav-btn-credentials'>
						<a href="{% url 'credentials' %}"><h5>Credentials</h5><i class="fas fa-plus"></i></a>
					</div>
					<div class='profile-nav-btn profile-nav-btn-wallet'>
						<a href="{% url 'indy_wallet' %}"><h5>Wallet</h5><i class="fas fa-plus"></i></a>
					</div>
				</div>
			</div>

			<div class='profile-tab-cont'>
				<div class='profile-search-input'>
					<input placeholder='Search'>
					<div class='btn btn-search-input'>
						<i class='fas fa-search'></i>
					</div>
				</div>

				{% block profile_content %}
				{% endblock %}
			</div>
		</div>
	</div>

	{% block profile_script %}
	{% endblock %}

{% endblock %}
