# Generated by Django 2.1.7 on 2019-04-09 00:51

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('indy_community', '0004_auto_20190328_0248'),
    ]

    operations = [
        migrations.AlterField(
            model_name='agentconnection',
            name='partner_name',
            field=models.CharField(max_length=60),
        ),
        migrations.AlterField(
            model_name='agentconversation',
            name='connection_partner_name',
            field=models.CharField(max_length=60),
        ),
        migrations.AlterField(
            model_name='indycredentialdefinition',
            name='creddef_handle',
            field=models.CharField(max_length=80),
        ),
        migrations.AlterField(
            model_name='indycredentialdefinition',
            name='creddef_name',
            field=models.CharField(max_length=80),
        ),
        migrations.AlterField(
            model_name='indycredentialdefinition',
            name='ledger_creddef_id',
            field=models.CharField(max_length=80, unique=True),
        ),
        migrations.AlterField(
            model_name='indyorganization',
            name='org_name',
            field=models.CharField(max_length=60, unique=True),
        ),
        migrations.AlterField(
            model_name='indyproofrequest',
            name='proof_req_name',
            field=models.CharField(max_length=400, unique=True),
        ),
        migrations.AlterField(
            model_name='indyschema',
            name='ledger_schema_id',
            field=models.CharField(max_length=80, unique=True),
        ),
        migrations.AlterField(
            model_name='indyschema',
            name='schema_name',
            field=models.CharField(max_length=80),
        ),
        migrations.AlterField(
            model_name='indyschema',
            name='schema_version',
            field=models.CharField(max_length=80),
        ),
        migrations.AlterField(
            model_name='indysession',
            name='wallet_name',
            field=models.CharField(blank=True, max_length=60, null=True),
        ),
        migrations.AlterField(
            model_name='indyuser',
            name='first_name',
            field=models.CharField(blank=True, max_length=80),
        ),
        migrations.AlterField(
            model_name='indywallet',
            name='wallet_name',
            field=models.CharField(max_length=60, unique=True),
        ),
    ]
