# Generated by Django 2.1.7 on 2019-03-27 13:32

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('indy_community', '0002_auto_20190319_2207'),
    ]

    operations = [
        migrations.CreateModel(
            name='IndyOrgRole',
            fields=[
                ('id', models.AutoField(auto_created=True, primary_key=True, serialize=False, verbose_name='ID')),
                ('name', models.CharField(max_length=40, unique=True)),
            ],
        ),
        migrations.AddField(
            model_name='indyorganization',
            name='role',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, to='indy_community.IndyOrgRole'),
        ),
        migrations.AddField(
            model_name='indyschema',
            name='roles',
            field=models.ManyToManyField(to='indy_community.IndyOrgRole'),
        ),
    ]
