{% load i18n %}

{% for credential in credentials %}
<div class='newsfeed-posted-cont'>
	<div class='newsfeed-posted-event-opps-cont'>
		<div>
			<a href=''>
				<h4>Credential</h4>
				<h6>Date Aquired</h6>
			</a>
		</div>
		<br>
		<div>
			<p>Schema: {{ credential.schema_id }}</p>
			<p>Cred Def: {{ credential.cred_def_id }}</p>
			<p>Referent: {{ credential.referent }}</p>
			<p>Attrs: {{ credential.attrs }}</p>
		</div>
	</div>
</div>
{% endfor %}

<div class='newsfeed-post-seperator'></div>