# Generated by Django 2.1.7 on 2019-04-13 00:01

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('indy_community', '0005_auto_20190409_0051'),
    ]

    operations = [
        migrations.RemoveField(
            model_name='agentconversation',
            name='connection_partner_name',
        ),
        migrations.RemoveField(
            model_name='agentconversation',
            name='wallet',
        ),
        migrations.AddField(
            model_name='agentconversation',
            name='connection',
            field=models.ForeignKey(default='1', on_delete=django.db.models.deletion.CASCADE, to='indy_community.AgentConnection', to_field='partner_name'),
            preserve_default=False,
        ),
        migrations.AlterField(
            model_name='agentconnection',
            name='partner_name',
            field=models.CharField(max_length=60, unique=True),
        ),
    ]
