# Generated by Django 2.1.7 on 2019-04-30 01:53

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('indy_community', '0007_auto_20190413_0125'),
    ]

    operations = [
        migrations.AddField(
            model_name='indyorganization',
            name='ico_url',
            field=models.CharField(blank=True, max_length=120, null=True),
        ),
    ]
