# Generated by Django 2.1.7 on 2019-03-28 02:48

from django.db import migrations, models
import django.db.models.deletion


class Migration(migrations.Migration):

    dependencies = [
        ('indy_community', '0003_auto_20190327_1332'),
    ]

    operations = [
        migrations.AlterField(
            model_name='indyorganization',
            name='wallet',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='wallet_org', to='indy_community.IndyWallet', to_field='wallet_name'),
        ),
        migrations.AlterField(
            model_name='indyuser',
            name='wallet',
            field=models.ForeignKey(blank=True, null=True, on_delete=django.db.models.deletion.CASCADE, related_name='wallet_user', to='indy_community.IndyWallet', to_field='wallet_name'),
        ),
    ]
