# Generated by Django 2.1.7 on 2019-04-30 03:31

from django.db import migrations, models


class Migration(migrations.Migration):

    dependencies = [
        ('indy_community', '0008_indyorganization_ico_url'),
    ]

    operations = [
        migrations.AddField(
            model_name='indyorganization',
            name='managed_wallet',
            field=models.BooleanField(default=True),
        ),
        migrations.AddField(
            model_name='indyuser',
            name='managed_wallet',
            field=models.BooleanField(default=True),
        ),
    ]
