{% load i18n %}
{% load static %}

<!DOCTYPE html>
<html>

<head>
	<meta charset="UTF-8">

	<title>{% block title %}Title{% endblock %}</title>

	{% block extra_head %}{% endblock %}
	
	<meta name="author" content="">
	<meta name="description" content="">
	<meta name="keywords" content="">
	<meta name="viewport" content="width=device-width,initial-scale=1">

	<link rel="stylesheet" href="{% static "/anon/style.css" %}" type="text/css">
	<link rel="stylesheet" href="{% static "/anon/navigations.css" %}" type="text/css">
	<link rel="shortcut icon" href="">
	<link rel="stylesheet" href="https://use.fontawesome.com/releases/v5.7.1/css/all.css" integrity="sha384-fnmOCqbTlWIlj8LyTjo7mOUStjsKC4pOpQbqyi7RrhN7udi9RwhKkMHpvLbHG9Sr" crossorigin="anonymous">
	<link href="https://fonts.googleapis.com/css?family=Montserrat:100,100i,200,200i,300,300i,400,400i,500,500i,600,600i,700,700i,800,800i,900,900i" rel="stylesheet">
	<link href="https://fonts.googleapis.com/css?family=Quicksand:300,400,500,700" rel="stylesheet">

	<script src="https://code.jquery.com/jquery-1.10.2.js"></script>
	<script src="//ajax.googleapis.com/ajax/libs/jquery/1.7.2/jquery.min.js" type="text/javascript"></script>
</head>

<body class='landing-page'>
	{% block main_content %}
    {% endblock %}
</body>

</html>
