{% extends request.session.INDY_PROFILE %}

{% load i18n %}
{% load static %}

{% block profile_content %}
	<div class='profile-tabs profile-tabs-connections'>
		<div id='profileconnections'>
			{% include "indy/pageIncludes/profileconnections.html" %}
		</div>
	</div>
{% endblock %}

{% block profile_script %}
	<script>
		$(document).ready(function() {
			$(".profile-tabs-connections").show();
			$(".profile-nav-btn-connections").addClass("active-profile-nav");
		});
	</script>
{% endblock %}
